Module engine

Module engine 

DMA engine managing TX/RX descriptor rings and frame I/O.

The engine owns the descriptor rings and data buffers, providing a high-level interface for frame transmission and reception without any register access. Register programming is handled by the caller.

Structs

DmaEngine
DMA engine with statically allocated buffers.